﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Chess.Test
{
    [TestClass]
    public class BishopTest
    {
        private const int DefaultBoardSize = 8;
        private Bishop Target { get; set; }

        [TestInitialize]
        public void BeforeEachTest()
        {
            Target = new Bishop();
        }

        [TestClass]
        public class GetMovesFrom : BishopTest
        {
            [TestMethod]
            public void ReturnsNonEmptyCollection()
            {
                Assert.IsTrue(Target.GetMovesFrom(new BoardCoordinate(1, 1), DefaultBoardSize).Any());
            }

            [TestMethod]
            public void Returns2_2For1_1()
            {
                Assert.IsTrue(Target.GetMovesFrom(new BoardCoordinate(1, 1), DefaultBoardSize).Any(c => c.X == 2 && c.Y == 2));
            }

            [TestMethod]
            public void Returns1_1For2_2()
            {
                Assert.IsTrue(Target.GetMovesFrom(new BoardCoordinate(2, 2), DefaultBoardSize).Any(c => c.X == 1 && c.Y == 1));
            }

            [TestMethod]
            public void Returns2_1For1_2()
            {
                Assert.IsTrue(Target.GetMovesFrom(new BoardCoordinate(1, 2), DefaultBoardSize).Any(c => c.X == 2 && c.Y == 1));
            }

            [TestMethod]
            public void Returns8_8For1_1()
            {
                Assert.IsTrue(Target.GetMovesFrom(new BoardCoordinate(1, 1), DefaultBoardSize).Any(c => c.X == 8 && c.Y == 8));
            }

            [TestMethod]
            public void Returns1_8For8_1()
            {
                Assert.IsTrue(Target.GetMovesFrom(new BoardCoordinate(8, 1), DefaultBoardSize).Any(c => c.X == 1 && c.Y == 8));
            }

            [TestMethod]
            public void ReturnsOnlyValidCoordinatesForDefaultBoardSizeFrom1_1()
            {
                Assert.IsFalse(Target.GetMovesFrom(new BoardCoordinate(1,1), DefaultBoardSize).Any(c => !c.IsValidCoordinateForBoardSize(DefaultBoardSize)));
            }
        }
        
    }
}
