﻿using System;
using System.Linq;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Chess.Test
{
    [TestClass]
    public class KingTest
    {
        private const int DefaultBoardSize = 8;
        private King Target { get; set; }
        private IEnumerable<BoardCoordinate> MovesFrom3_3 { get; set; }

        [TestInitialize]
        public void BeforeEachTest()
        {
            Target = new King();
            MovesFrom3_3 = Target.GetMovesFrom(new BoardCoordinate(3, 3), DefaultBoardSize);
        }

        [TestClass]
        public class GetMovesFrom : KingTest
        {
            [TestMethod]
            public void Returns2_1For1_1()
            {
                Assert.IsTrue(Target.GetMovesFrom(new BoardCoordinate(1, 1), DefaultBoardSize).Any(m => m.X == 2 && m.Y == 1));
            }

            [TestMethod]
            public void Returns2_2For1_1()
            {
                Assert.IsTrue(Target.GetMovesFrom(new BoardCoordinate(1, 1), DefaultBoardSize).Any(m => m.X == 2 && m.Y == 2));
            }

            [TestMethod]
            public void Returns3_4For3_3()
            {
                Assert.IsTrue(MovesFrom3_3.Any(m => m.X == 3 && m.Y == 4));
            }

            [TestMethod]
            public void Returns3_2For3_3()
            {
                Assert.IsTrue(MovesFrom3_3.Any(m => m.X == 3 && m.Y == 2));
            }

            [TestMethod]
            public void Returns2_3For3_3()
            {
                Assert.IsTrue(MovesFrom3_3.Any(m => m.X == 2 && m.Y == 3));
            }

            [TestMethod]
            public void DoesNotReturn0_0For1_1()
            {
                Assert.IsFalse(Target.GetMovesFrom(new BoardCoordinate(1, 1), DefaultBoardSize).Any(m => m.X == 0 && m.Y == 0));
            }

            [TestMethod]
            public void ReturnsOnlyValidCoordinatesForDefaultBoardSizeFrom1_1()
            {
                Assert.IsFalse(Target.GetMovesFrom(new BoardCoordinate(1, 1), DefaultBoardSize).Any(c => !c.IsValidCoordinateForBoardSize(DefaultBoardSize)));
            }
        }
        
    }
}
