﻿using System;
using System.Linq;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Chess.Test
{
    [TestClass]
    public class PawnTest
    {
        private const int DefaultBoardSize = 8;
        private readonly BoardCoordinate DefaultCoordinate = new BoardCoordinate(4,2);
        private Pawn Target { get; set; }
        private IEnumerable<BoardCoordinate> MovesFromDefaultCoordinate { get; set; }

        [TestInitialize]
        public void BeforeEachTest()
        {
            Target = new Pawn();
            MovesFromDefaultCoordinate = Target.GetMovesFrom(DefaultCoordinate, DefaultBoardSize);
        }

        [TestClass]
        public class GetMovesFrom : PawnTest
        {
            [TestMethod]
            public void Returns4_3AsOneResultWhenPassed4_2()
            {
                Assert.IsTrue(MovesFromDefaultCoordinate.Any(c => c.Equals(new BoardCoordinate(4, 3))));
            }

            [TestMethod]
            public void Returns4_4AsOneResultWhenPassed4_2WhenPieceHasNotMoved()
            {
                Assert.IsTrue(MovesFromDefaultCoordinate.Any(c => c.Equals(new BoardCoordinate(4, 4))));
            }

            [TestMethod]
            public void DoesNotReturn4_4AsOneResultWhenPassed4_2IfPieceHasAlreadyMoved()
            {
                Target.HasMoved = true;

                Assert.IsFalse(MovesFromDefaultCoordinate.Any(c => c.Equals(new BoardCoordinate(4, 4))));
            }
        }

    }
}
